"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LegacyHatchetClient = void 0;
const zod_1 = require("zod");
const config_loader_1 = require("../../util/config-loader");
const event_client_1 = require("../event/event-client");
const dispatcher_client_1 = require("../dispatcher/dispatcher-client");
const admin_client_1 = require("../admin/admin-client");
const nice_grpc_1 = require("nice-grpc");
const worker_1 = require("../worker");
const hatchet_logger_1 = require("./hatchet-logger");
const grpc_helpers_1 = require("../../util/grpc-helpers");
const client_config_1 = require("./client-config");
const child_listener_client_1 = require("../listeners/run-listener/child-listener-client");
const rest_1 = __importDefault(require("../rest"));
const cron_client_1 = require("./features/cron-client");
const schedule_client_1 = require("./features/schedule-client");
const durable_listener_client_1 = require("../listeners/durable-listener/durable-listener-client");
class LegacyHatchetClient {
    constructor(config, options, axiosOpts, runs) {
        // Initializes a new Client instance.
        // Loads config in the following order: config param > yaml file > env vars
        var _a;
        this.runs = runs;
        const loaded = config_loader_1.ConfigLoader.loadClientConfig(config, {
            path: options === null || options === void 0 ? void 0 : options.config_path,
        });
        try {
            const valid = client_config_1.ClientConfigSchema.parse(loaded);
            let logConstructor = config === null || config === void 0 ? void 0 : config.logger;
            if (logConstructor == null) {
                logConstructor = hatchet_logger_1.DEFAULT_LOGGER;
            }
            this.config = Object.assign(Object.assign({}, valid), { logger: logConstructor });
        }
        catch (e) {
            if (e instanceof zod_1.z.ZodError) {
                throw new Error(`Invalid client config: ${e.message}`);
            }
            throw e;
        }
        this.credentials =
            (_a = options === null || options === void 0 ? void 0 : options.credentials) !== null && _a !== void 0 ? _a : config_loader_1.ConfigLoader.createCredentials(this.config.tls_config);
        const clientFactory = (0, nice_grpc_1.createClientFactory)().use((0, grpc_helpers_1.addTokenMiddleware)(this.config.token));
        this.tenantId = this.config.tenant_id;
        this.api = (0, rest_1.default)(this.config.api_url, this.config.token, axiosOpts);
        this.event = new event_client_1.EventClient(this.config, (0, grpc_helpers_1.channelFactory)(this.config, this.credentials), clientFactory);
        this.dispatcher = new dispatcher_client_1.DispatcherClient(this.config, (0, grpc_helpers_1.channelFactory)(this.config, this.credentials), clientFactory);
        this.listener = new child_listener_client_1.RunListenerClient(this.config, (0, grpc_helpers_1.channelFactory)(this.config, this.credentials), clientFactory, this.api);
        this.admin = new admin_client_1.AdminClient(this.config, (0, grpc_helpers_1.channelFactory)(this.config, this.credentials), clientFactory, this.api, this.tenantId, this.listener, this.runs);
        this.durableListener = new durable_listener_client_1.DurableListenerClient(this.config, (0, grpc_helpers_1.channelFactory)(this.config, this.credentials), clientFactory, this.api);
        this.logger = this.config.logger('HatchetClient', this.config.log_level);
        this.logger.debug(`Initialized HatchetClient`);
        // Feature Clients
        this.cron = new cron_client_1.CronClient(this.tenantId, this.config, this.api, this.admin);
        this.schedule = new schedule_client_1.ScheduleClient(this.tenantId, this.config, this.api, this.admin);
    }
    static init(config, options, axiosConfig) {
        return new LegacyHatchetClient(config, options, axiosConfig);
    }
    // @deprecated
    run(workflow) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.warn('HatchetClient.run is deprecated and will be removed in a future release. Use HatchetClient.worker and Worker.start instead.');
            const worker = yield this.worker(workflow);
            worker.start();
            return worker;
        });
    }
    worker(workflow, opts) {
        return __awaiter(this, void 0, void 0, function* () {
            const name = typeof workflow === 'string' ? workflow : workflow.id;
            let options = {
                name,
            };
            if (typeof opts === 'number') {
                this.logger.warn('@deprecated maxRuns param is deprecated and will be removed in a future release in favor of WorkerOpts');
                options = Object.assign(Object.assign({}, options), { maxRuns: opts });
            }
            else {
                options = Object.assign(Object.assign({}, options), opts);
            }
            const worker = new worker_1.V0Worker(this, options);
            if (typeof workflow !== 'string') {
                yield worker.registerWorkflow(workflow);
                return worker;
            }
            return worker;
        });
    }
    webhooks(workflows) {
        // TODO v1 workflows
        const worker = new worker_1.V0Worker(this, {
            name: 'webhook-worker',
        });
        return worker.getHandler(workflows);
    }
}
exports.LegacyHatchetClient = LegacyHatchetClient;
